import os
import sys
from .. import log
try:
    ABLE_BUILD_FLAG = True
    from grpc_tools import protoc
except ImportError:
    ABLE_BUILD_FLAG = False
import tempfile


def build(
    *proto_files: str,
    proto_path: str,
    output_dir: str = 'pb',
    output_type: list[str] = [
        "python", "grpclib_python", "mypy"],
        other_include: list[str] = [],):
    path = os.path.abspath(proto_path)
    outputpath = output_dir.lstrip("./")
    global_build_flag = True
    if (not os.path.exists(path)):
        global_build_flag = False
        if (not os.path.exists(outputpath)):
            log.log_compile_global_fault(
                "Compiled Proto not found")
            sys.exit(210)
    if (not os.path.exists(outputpath)):
        os.makedirs(outputpath)
    if (not os.path.exists(outputpath+os.sep+"__init__.py")):
        with open(outputpath+os.sep+"__init__.py", "w") as file:
            file.write("")
    if (not os.path.exists(outputpath+os.sep+".grpcEcache")):
        with open(outputpath+os.sep+".grpcEcache", "w") as file:
            file.write(";")
    with open(outputpath+os.sep+".grpcEcache", "r") as file:
        info = {i.split(";")[0]: i.split(";")[1]
                for i in file.read().rstrip("\n").split("\n")}
    if global_build_flag:
        if (ABLE_BUILD_FLAG is False):
            log.log_compile_global_fault(
                "grpcio-tools Not found")
            sys.exit(210)
        for protoname in proto_files:
            if (not os.path.exists(path+os.sep+protoname)):
                log.log_compile_global_fault(
                    f'File "{protoname}" not found')
            file_lasttime = os.path.getmtime(path+os.sep+protoname)
            if (info.get(path+os.sep+protoname, "") == str(file_lasttime)):
                continue

            args = [
                'grpc_tools.protoc',
                '-I=.',
                '--proto_path='+path
            ]
            args.extend(['-I='+i for i in other_include])
            args.extend(['--'+i+'_out=./'+outputpath for i in output_type])
            args.append(protoname)

            log.log_compile(protoname)

            # 保存原始的文件描述符
            original_stdout_fd = os.dup(sys.stdout.fileno())
            original_stderr_fd = os.dup(sys.stderr.fileno())

            # 创建临时文件
            with tempfile.NamedTemporaryFile(mode='w+', delete=False) as stdout_temp, \
                    tempfile.NamedTemporaryFile(mode='w+', delete=False) as stderr_temp:

                try:
                    # 重定向标准输出和标准错误到临时文件
                    os.dup2(stdout_temp.fileno(), sys.stdout.fileno())
                    os.dup2(stderr_temp.fileno(), sys.stderr.fileno())

                    # 执行 protoc
                    result = protoc.main(args)  # type: ignore

                finally:
                    # 恢复原始的文件描述符
                    os.dup2(original_stdout_fd, sys.stdout.fileno())
                    os.dup2(original_stderr_fd, sys.stderr.fileno())
                    os.close(original_stdout_fd)
                    os.close(original_stderr_fd)

                    # 确保所有内容都写入临时文件
                    sys.stdout.flush()
                    sys.stderr.flush()

            with open(stderr_temp.name, 'r') as f:
                stderr_output = f.read()

            # 删除临时文件
            os.unlink(stdout_temp.name)
            os.unlink(stderr_temp.name)

            # 处理输出
            if result != 0:
                stderr_output = stderr_output.strip('\n')
                for i in stderr_output.split("\n"):
                    if (" " in i):
                        file_name = i.split(" ")[0][:-1]
                        if (":" not in file_name):
                            log.log_compile_global_fault(
                                i)
                        else:
                            file_more = i[len(file_name)+2:]

                            log.log_compile_file_fault(
                                file_name, file_more)
                    else:
                        log.log_compile_global_fault(
                            i)

                sys.exit(212)
            # if "grpc_python" in output_type:
            #     patched_grpcname, patched_grpcnameorigin = patch_grpcname(
            #         protoname)
            #     with open(outputpath+os.sep+patched_grpcname, "r") as file:
            #         data = file.read()
            #     with open(outputpath+os.sep+patched_grpcname, "w") as file:
            #         file.write(data.replace(
            #             f"import {patched_grpcnameorigin}_pb2", f"from . import {patched_grpcnameorigin}_pb2"))

            info[path+os.sep+protoname] = str(file_lasttime)
        with open(outputpath+os.sep+".grpcEcache", "w") as file:
            file.write("\n".join([f"{k};{v}" for k, v in info.items()]))



import inspect
import random
from abc import ABC, abstractmethod
from typing import Type, TypeVar, Any, get_type_hints

T = TypeVar('T', bound=ABC)


def create_implementation(_, abstract_class):
    """
    动态创建实现抽象类的类

    Args:
        abstract_class: 抽象类

    Returns:
        新创建的类
    """

    # 创建方法字典
    methods = {}

    rand_class_name = "VC_"+str(random.randint(1000000, 9999999))

    class_code = f"class {rand_class_name}(abstract_class):\n"

    # 获取所有抽象方法
    for name, method in inspect.getmembers(abstract_class, predicate=inspect.isfunction):
        if getattr(method, '__isabstractmethod__', False):
            # 获取返回类型
            hints = get_type_hints(method)
            return_type = hints.get('return', None)

            # 创建默认实现
            def make_method(name, return_type):
                def default_impl(self, *args, **kwargs):
                    return None
                # 复制原始方法的元数据
                default_impl.__module__ = method.__module__
                default_impl.__qualname__ = method.__qualname__
                default_impl.__doc__ = method.__doc__
                default_impl.__annotations__ = method.__annotations__
                default_impl.__name__ = name
                default_impl.__signature__ = inspect.signature(  # type: ignore
                    method)  # type: ignore

                return default_impl

            methods[name] = make_method(name, return_type)
            sig = inspect.signature(method)
            param_parts = []
            for param_name, param in sig.parameters.items():
                if param_name == 'self':
                    param_parts.append('self')
                    continue
                if param.default is param.empty:
                    param_parts.append(param_name)
                else:
                    default_repr = repr(param.default)
                    param_parts.append(f"{param_name}={default_repr}")
                if param.kind == param.VAR_POSITIONAL:
                    param_parts[-1] = f"*{param_name}"
                elif param.kind == param.VAR_KEYWORD:
                    param_parts[-1] = f"**{param_name}"

            # 组合参数字符串
            params_str = ", ".join(param_parts)
            def_string = f"    def {name}({params_str}):\n        return None\n"

            class_code += def_string

    # 动态创建类
    return type(abstract_class.__name__, (abstract_class,), methods), rand_class_name, class_code



import inspect

from . import utils
from typing import Callable


def set_func_io(self, func: Callable, method: str, stream=None):
    if (":" in method):
        obj = self.services[self.service_to_class[method.split(":")[
            0]]]
        methodname = method.split(":")[1]
    else:
        obj = self.services[0]
        methodname = method
    func_inspect = inspect.signature(func)
    build_code = "(lambda request,func,stream=None: func("
    for name, _ in func_inspect.parameters.items():
        if name == "stream":
            continue
        build_code += f"{name}=request.{name},"
    if (stream != None):
        build_code += f"stream=stream,"
    build_code += "))"
    request_solve = eval(build_code)

    # 展开输出
    method_obj = getattr(obj, methodname)
    out_typename = utils.extract_class_from_type_string(
        method_obj.__annotations__["stream"], 1)
    if (out_typename[0].endswith("_pb2")):
        out_typename[0] = out_typename[0][:-4]  # type: ignore
    outtype_obj = getattr(self.proto, "manifest_" +
                          out_typename[0])[out_typename[1]]
    outtype_realobj = getattr(getattr(self.proto, "pb_" +
                                      out_typename[0]), out_typename[1])
    vals = utils.get_val_from_pb(outtype_obj)

    outtype_obj.__instancecheck__ = (lambda *args, **kwargs: True)
    outtype_obj.__subclasscheck__ = (lambda *args, **kwargs: True)

    # 构建输出映射
    build_code_out = "(lambda response,type_f: type_f("
    for name, _ in vals:
        build_code_out += f"{name}=response.get(\"{name}\",None),"
    build_code_out += "))"
    self.prod_gen_code.append({
        "name": method,
        "code": (build_code, build_code_out),
        "outtype": (out_typename[0], out_typename[1])
    })
    response_solve = eval(build_code_out)
    return obj, methodname, request_solve, outtype_realobj, response_solve



import builtins
from ..log import log_print
import sys


def print(*values,
          sep: str = ' ',
          end: str = '\n',
          file=None,
          flush: bool = False) -> None:

    # 将所有值转换为字符串并用 sep 连接
    output = sep.join(str(value) for value in values)

    # 添加结束字符
    output += end

    # 写入到指定的文件对象
    if file is not None:
        file.write(output)
        if flush:
            file.flush()
        return

    for i in output.rstrip("\n").split("\n"):
        log_print(i)


def fix():
    builtins.print = print



#!/usr/bin/env python
import argparse
import importlib.util
import os
import sys
try:
    from ..product_mode import run as build
except ImportError:
    build = None


def import_from_path(path: str):
    """从路径导入对象，格式为 file_path:object_name"""
    if ":" not in path:
        raise ValueError(
            f"Invalid format: {path}. Expected format: file_path:object_name")

    file_path, object_name = path.split(":", 1)

    # 确保文件路径是绝对路径
    if not os.path.isabs(file_path):
        file_path = os.path.abspath(file_path)

    # 检查文件是否存在
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"File not found: {file_path}")

    # 导入模块
    module_name = os.path.basename(file_path).replace(".py", "")
    spec = importlib.util.spec_from_file_location(module_name, file_path)
    if spec is None or spec.loader is None:
        raise ImportError(f"Could not import module from {file_path}")

    module = importlib.util.module_from_spec(spec)
    sys.modules[module_name] = module
    spec.loader.exec_module(module)

    # 获取对象
    if not hasattr(module, object_name):
        raise AttributeError(
            f"Object '{object_name}' not found in {file_path}")

    return getattr(module, object_name)


def run_command(args):
    """执行run命令的逻辑"""
    try:
        app = import_from_path(args.app_path)
        app.run(host=args.host, port=args.port, loop=args.loop)

    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)


def build_command(args):
    """执行build命令的逻辑"""
    try:
        app = import_from_path(args.app_path)
        if (build != None):
            build(app, args.dist, args.sources)
        else:
            print("build is not useable in product mode!")
            sys.exit(1)

    except Exception as e:
        print(f"Error during build: {e}", file=sys.stderr)
        sys.exit(1)


def main():
    """主函数，设置命令行参数解析"""
    parser = argparse.ArgumentParser(
        description="Command line tool for running and building applications"
    )
    subparsers = parser.add_subparsers(dest="command", help="Commands")

    # 创建run命令的子解析器
    run_parser = subparsers.add_parser("run", help="Run an application")
    run_parser.add_argument(
        "app_path",
        type=str,
        help="Path to the application in the format file_path:app_name (e.g., main.py:app)"
    )
    run_parser.add_argument(
        "--host",
        type=str,
        default="0.0.0.0",
        help="Host to bind the server to (default: 0.0.0.0)"
    )
    run_parser.add_argument(
        "--port",
        type=int,
        default=50051,
        help="Port to bind the server to (default: 50051)"
    )
    run_parser.add_argument(
        "--loop",
        type=str,
        choices=["auto", "asyncio", "uvloop", "proactor"],
        default="auto",
        help="Event loop implementation to use (default: auto)"
    )

    # 创建build命令的子解析器
    build_parser = subparsers.add_parser("build", help="Build an application")
    build_parser.add_argument(
        "app_path",
        type=str,
        help="Path to the application in the format file_path:app_name (e.g., main.py:app)"
    )
    build_parser.add_argument(
        "--dist",
        type=str,
        default="dist",
        help="Directory where build artifacts will be saved"
    )
    build_parser.add_argument(
        "sources",
        nargs="*",
        help="Source files or directories to include in the build"
    )

    args = parser.parse_args()

    if args.command is None:
        parser.print_help()
        sys.exit(1)

    if args.command == "run":
        run_command(args)
    elif args.command == "build":
        build_command(args)
    else:
        parser.print_help()
        sys.exit(1)


if __name__ == "__main__":
    main()



import os
from ..engine import GRPCLess
import shutil
from .. import log

NOWPATH = os.path.abspath(os.path.dirname(
    __file__)+os.sep+".."+os.sep+"..")
PKGNAME = "grpcless"

REWRITE_ENGINE_SETIO_TEMPLATE = """
def set_func_io(self, func, method: str, stream=None):
"""
REWRITE_ENGINE_FILLABC_HEAD_TEMPLATE = """
def create_implementation(srv, abstract_class):
"""

REWRITE_PROTO_HEAD_TEMPLATE = """
from typing import Any
"""
REWRITE_PROTO_CLASS_TEMPLATE = """
class Proto():
    def __init__(self,
                 *proto_files: str,
                 proto_path: str,
                 output_dir: str = 'pb',
                 output_type: list[str] = [
                     "python", "grpclib_python", "mypy"],
                 other_include: list[str] = [],
                 ):
"""
REWRITE_PROTO_TAIL_TEMPLATE = """
    def get_service(self, proto: str):
        protofile, service_name = proto.split(":")
        return getattr(self.import_protos[protofile][1], service_name+"Base")
"""


def build(app: GRPCLess, dist: str = "dist"):
    log.log_build_start()
    if (os.path.exists(dist)):
        log.log_build("Remove old dist")
        shutil.rmtree(dist)
    log.log_build("Copy lib")
    shutil.copytree(NOWPATH+os.sep+PKGNAME, dist+os.sep+PKGNAME)
    log.log_build("Create setio")
    rewrite_ENGINE_SETIO_TEMPLATE = """
from . import utils
def set_func_io(self, func, method: str, stream=None):
"""
    for i in app.prod_gen_code:
        method = i["name"]
        if (":" in method):
            obj = f"self.services[self.service_to_class[{repr(method)}]]"
            methodname = method.split(":")[1]
        else:
            obj = f"self.services[0]"
            methodname = method
        rewrite_ENGINE_SETIO_TEMPLATE += f"    if method == '{i["name"]}':\n"
        rewrite_ENGINE_SETIO_TEMPLATE += f"        return {obj},{repr(methodname)},{i["code"][0]},self.proto.pb_{i["outtype"][0]}.{i["outtype"][1]},{i["code"][1]}\n"
    rewrite_ENGINE_SETIO_TEMPLATE += "    raise ValueError('method not found')"
    with open(dist+os.sep+PKGNAME+os.sep+"engine"+os.sep+"setio.py", "w") as f:
        f.write(rewrite_ENGINE_SETIO_TEMPLATE)

    log.log_build("Create fillabc")
    code_fill_abc = REWRITE_ENGINE_FILLABC_HEAD_TEMPLATE
    for i in app._prod_v_class_code:
        srv_name = i[0]
        cls_name = i[1]
        cls_code = i[2]
        code_fill_abc += f"    if(srv=={repr(srv_name)}):\n"
        code_fill_abc += '\n'.join([" "*8+j for j in cls_code.split("\n")])
        code_fill_abc += f"\n        return {cls_name},{repr(cls_name)},''\n"

    with open(dist+os.sep+PKGNAME+os.sep+"engine"+os.sep+"fill_abc.py", "w") as f:
        f.write(code_fill_abc)

    log.log_build("Copy proto")
    # copy proto
    proto_dist_path = app.proto._output_dir
    for i in os.listdir(proto_dist_path):
        if (i.endswith(".py") and i != "__init__.py"):
            shutil.copy(proto_dist_path+os.sep+i, dist+os.sep+i)

    log.log_build("Create protoobj")
    code_head = REWRITE_PROTO_HEAD_TEMPLATE
    code_class_first = REWRITE_PROTO_CLASS_TEMPLATE
    code_class_first += "        self.import_protos={\n"
    code_class_second = ""
    cnt = 0
    for i in app.proto._prod_dist:
        file_name = i[0]
        base_name = i[1]
        proto_module = i[2][0]
        proto_grpc_module = i[2][1]
        code_head += f"import {proto_module} as pb_module_gen_{cnt}\n"
        code_head += f"import {proto_grpc_module} as pbgrpc_module_gen_{cnt}\n"
        code_class_first += f"            {repr(file_name)}: (pb_module_gen_{cnt}, pbgrpc_module_gen_{cnt}, {{}}),\n"
        code_class_second += f"        self.pb_{base_name}=pb_module_gen_{cnt}\n"
        code_class_second += f"        self.grpc_{base_name}=pbgrpc_module_gen_{cnt}\n"
        code_class_second += f"        self.manifest_{base_name}={{}}\n"
        cnt += 1
    code_class_first += "        }\n"
    code = code_head + code_class_first + \
        code_class_second + REWRITE_PROTO_TAIL_TEMPLATE
    with open(dist+os.sep+PKGNAME+os.sep+"proto"+os.sep+"__init__.py", "w") as f:
        f.write(code)

    # 删除不必要的组件
    shutil.rmtree(dist+os.sep+PKGNAME+os.sep+"product_mode")
    shutil.rmtree(dist+os.sep+PKGNAME+os.sep+"import_fixer")

    log.log_build("Finish level")



import sys
import importlib
import os
from .. import log
from . import utils


ENVCLEAN_LIST = [
    "DESCRIPTOR"
]


def init_path(path: str) -> None:
    log.log_import_access(path)
    sys.path.append(os.path.abspath(path))


def import_proto(path: str, proto: str) -> tuple:
    proto_pkg = utils.patch_pbname(proto)[1]+"_pb2"
    grpcproto_pkg = utils.patch_grpcname(proto)[1]+"_grpc"
    log.log_import_proto(proto)
    protoobj = importlib.import_module(proto_pkg)
    grpcprotoobj = importlib.import_module(grpcproto_pkg)
    return protoobj, grpcprotoobj, import_pyi(path, proto), (proto_pkg, grpcproto_pkg)


def import_pyi(path: str, proto: str) -> object:
    proto_pkg = path+os.sep+utils.patch_pbname(proto)[0]+"i"
    with open(proto_pkg) as f:
        code = f.read()
    env = {}
    try:
        exec(code, env, env)
    except Exception:
        pass
    for i in list(env.keys()):
        if i.startswith("_") or i in ENVCLEAN_LIST:
            del env[i]
    return env




import os
import platform
import sys


__all__ = [
    "RESET",
    "FG_BLACK",
    "FG_RED",
    "FG_GREEN",
    "FG_YELLOW",
    "FG_BLUE",
    "FG_MAGENTA",
    "FG_CYAN",
    "FG_WHITE",
    "FG_DEFAULT",
    "BG_BLACK",
    "BG_RED",
    "BG_GREEN",
    "BG_YELLOW",
    "BG_BLUE",
    "BG_MAGENTA",
    "BG_CYAN",
    "BG_WHITE",
    "BG_DEFAULT",
    "FG_GREY",
    "BG_GREY",
]


def supports_color():
    if os.getenv('ANSI_COLORS_DISABLED') is not None:
        return False
    if not sys.stdout.isatty():
        return False
    plat = platform.system()
    if plat == 'Windows':
        if int(platform.release()) >= 10:
            try:
                build = int(platform.version().split('.')[-1])
                if build >= 14931:
                    return True
            except (ValueError, IndexError):
                pass
        return os.getenv('ANSICON') is not None or \
            os.getenv('WT_SESSION') is not None or \
            os.getenv('ConEmuANSI') == 'ON' or \
            os.getenv('TERM') == 'xterm'
    # 对于 Linux, macOS 等类 Unix 系统，大多数终端支持颜色
    return True


if supports_color():
    RESET = '\033[0m'
    FG_BLACK = '\033[30m'
    FG_RED = '\033[31m'
    FG_GREEN = '\033[32m'
    FG_YELLOW = '\033[33m'
    FG_BLUE = '\033[34m'
    FG_MAGENTA = '\033[35m'
    FG_CYAN = '\033[36m'
    FG_WHITE = '\033[37m'
    FG_DEFAULT = '\033[39m'
    BG_BLACK = '\033[40m'
    BG_RED = '\033[41m'
    BG_GREEN = '\033[42m'
    BG_YELLOW = '\033[43m'
    BG_BLUE = '\033[44m'
    BG_MAGENTA = '\033[45m'
    BG_CYAN = '\033[46m'
    BG_WHITE = '\033[47m'
    BG_DEFAULT = '\033[49m'
    FG_GREY = '\033[90m'
    BG_GREY = '\033[100m'
else:
    RESET = ''
    FG_BLACK = ''
    FG_RED = ''
    FG_GREEN = ''
    FG_YELLOW = ''
    FG_BLUE = ''
    FG_MAGENTA = ''
    FG_CYAN = ''
    FG_WHITE = ''
    FG_DEFAULT = ''
    BG_BLACK = ''
    BG_RED = ''
    BG_GREEN = ''
    BG_YELLOW = ''
    BG_BLUE = ''
    BG_MAGENTA = ''
    BG_CYAN = ''
    BG_WHITE = ''
    BG_DEFAULT = ''
    FG_GREY = ''
    BG_GREY = ''



from importlib import metadata
from typing import Callable, Coroutine, Any
import traceback

from .. import log
from grpclib.exceptions import GRPCError, StreamTerminatedError
from grpclib.const import Status


def ErrorTraceMiddleware(func: Callable[[Any], Coroutine]) -> Callable:
    async def warpper(request) -> Coroutine:
        try:
            ret = await func(request)
        except GRPCError as e:
            raise e
        except StreamTerminatedError as e:
            raise e
        except Exception as e:
            error_str = traceback.format_exc().strip("\n")
            for i in error_str.split("\n"):
                log.log_error("    "+i)

            raise GRPCError(
                status=Status.INTERNAL,
                message="Server Error: "+str(e),
                details={
                    "fullname": repr(e),
                    "middleware": "grpce.middleware.ErrorTraceMiddleware",
                    "server": "grpcE"
                }
            )
        return ret
    return warpper



import sys
from importlib.machinery import ModuleSpec
import types
import importlib
from importlib.abc import Loader


class LazyLoader(Loader):
    def __init__(self, module_name):
        self.module_name = module_name

    def create_module(self, spec):
        """创建模块对象"""
        # 先创建一个代理模块
        mod = LazyModule(self.module_name, self.module_name)
        import_pb_list.append(mod)
        return mod

    def exec_module(self, module):
        pass


class LazyModule(types.ModuleType):
    """代理模块，在访问属性时才加载真实模块"""

    def __init__(self, name, real_mod: str):
        self.__name__ = name
        self.__package__ = None
        self.__spec__ = None
        self.real_module = None
        self.real_module_name = real_mod

    def __set_real_module__(self):
        importlib.reload(sys.modules[self.real_module_name])


class ImportFixer:
    def find_spec(self, name: str, path, target=None):
        if (name.endswith("_pb2") or name.endswith("_grpc")):
            # 创建带有延迟加载器的 ModuleSpec
            loader = LazyLoader(name)
            spec = ModuleSpec(name, loader)
            return spec
        return None


import_pb_list: list[LazyModule] = []


fixer = ImportFixer()


def fix():
    sys.meta_path.insert(0, fixer)


def real_import():
    global import_pb_list
    sys.meta_path.remove(fixer)
    for i in import_pb_list:
        i.__set_real_module__()
    import_pb_list = []



from grpclib.server import Stream as gStream
from operator import methodcaller
from ..proto import Proto
from ..middleware import LogMiddleware, LogStreamMiddleware, ErrorTraceMiddleware
from grpclib.server import Server
import asyncio
from typing import Literal, AsyncGenerator
import sys
from typing import Any, Callable
try:
    UVLOOPMODE = True
    import uvloop  # type: ignore
except:
    UVLOOPMODE = False
from . import fill_abc
from . import setio
from .. import log
from .. import printfix
from .. import stream
import asyncio


async def default_server_life(server):
    yield


class GRPCLess():
    def __init__(self, proto: Proto, *service: str, life: Callable[[Any], AsyncGenerator[None, None]] = default_server_life, global_middleware: list[Callable] = [ErrorTraceMiddleware]):
        self.proto = proto
        self.service_to_class = {
            j.split(":")[0]: i for i, j in enumerate(service)}
        self.services = []
        self._prod_v_class_code = []
        for i in service:
            obj, class_name, class_code = fill_abc.create_implementation(i,
                                                                         proto.get_service(i))
            self.services.append(obj)
            self._prod_v_class_code.append((i, class_name, class_code))
        self.service_from_proto = [i for i in service]
        self.life = life
        self.global_middleware = global_middleware
        self.prod_gen_code = []

    def run(self,
            host: str = "0.0.0.0",
            port: int = 50051,
            loop: Literal["auto", "asyncio", "uvloop", "proactor"] = "auto",
            fix_print: bool = True,
            ):
        if (fix_print):
            printfix.fix()

        if loop == "auto":
            if UVLOOPMODE:
                loop = "uvloop"
            elif sys.platform == 'win32':
                loop = "proactor"
            else:
                loop = "asyncio"
        if loop == "uvloop" and UVLOOPMODE:
            uvloop.install()  # type: ignore
        elif loop == "proactor":
            asyncio.set_event_loop_policy(
                asyncio.WindowsProactorEventLoopPolicy())  # type: ignore

        async def start_server():
            server = Server([i() for i in self.services])
            async for _ in self.life(server):
                await server.start(host, port)
                await server.wait_closed()

        log.log_start_server(host, port, loop)

        asyncio.run(start_server())

    def add_middleware(self, middleware: Callable):
        self.global_middleware.append(middleware)

    def __set_func_io(self, func: Callable, method: str, stream=None):
        return setio.set_func_io(self, func, method, stream)

    def request(self, method: str, *, middleware: list[Callable] = [LogMiddleware], use_global_middleware: bool = True):
        def decorator(func):
            # 构建输入映射
            obj, methodname, request_solve, outtype_realobj, response_solve = self.__set_func_io(
                func, method)

            # 构建中间件
            async def run(request):
                return response_solve(await request_solve(request, func) or {}, outtype_realobj)
            if (use_global_middleware):
                for i in reversed(self.global_middleware):
                    run = i(run)
            for i in reversed(middleware):
                run = i(run)

            async def wrapper(self, stream):
                request = await stream.recv_message()
                ret = await run(request)
                await stream.send_message(ret)
            setattr(obj, methodname, wrapper)
            return wrapper
        return decorator

    def stream(self, method: str, *, middleware: list[Callable] = [LogStreamMiddleware], use_global_middleware: bool = True):
        def decorator(func):
            # 构建输入映射
            obj, methodname, request_solve, outtype_realobj, response_solve = self.__set_func_io(
                func, method)

            async def run(request):
                streams = stream.Stream(
                    request, (lambda x: response_solve(x or {}, outtype_realobj)), methodname)
                ret = await func(streams)
                await asyncio.sleep(0.01)  # 不加这个会 13 INTERNAL
                # try:
                #     await streams.close()
                # except:
                #     pass
                return ret
            # 构建中间件
            if (use_global_middleware):
                for i in reversed(self.global_middleware):
                    run = i(run)
            for i in reversed(middleware):
                run = i(run)

            async def wrapper(self, stream):
                stream.req_name = methodname
                return await run(stream)
            setattr(obj, methodname, wrapper)
            return wrapper
        return decorator

    def server_stream(self, method: str, *, middleware: list[Callable] = [LogStreamMiddleware], use_global_middleware: bool = True):
        def decorator(func):
            # 构建输入映射
            obj, methodname, request_solve, outtype_realobj, response_solve = self.__set_func_io(
                func, method, True)

            async def run(request: gStream):
                req_ctx = request
                req = await req_ctx.recv_message()
                streams = stream.ServerStream(
                    req_ctx, (lambda x: response_solve(x or {}, outtype_realobj)), methodname)
                ret = await request_solve(req, func, streams)
                await streams.close()
                await asyncio.sleep(0.01)  # 不加这个会 13 INTERNAL
                return ret
            # 构建中间件
            if (use_global_middleware):
                for i in reversed(self.global_middleware):
                    run = i(run)
            for i in reversed(middleware):
                run = i(run)

            async def wrapper(self, stream):
                stream.req_name = methodname
                return await run(stream)
            setattr(obj, methodname, wrapper)
            return wrapper
        return decorator

    def client_stream(self, method: str, *, middleware: list[Callable] = [LogStreamMiddleware], use_global_middleware: bool = True):

        def decorator(func):
            # 构建输入映射
            obj, methodname, request_solve, outtype_realobj, response_solve = self.__set_func_io(
                func, method, True)

            async def run(request: gStream):
                req_ctx = request
                streams = stream.ClientStream(
                    req_ctx, (lambda x: response_solve(x or {}, outtype_realobj)), methodname)
                ret = response_solve(await func(streams) or {}, outtype_realobj)
                await request.send_message(ret)
                return None
            # 构建中间件
            if (use_global_middleware):
                for i in reversed(self.global_middleware):
                    run = i(run)
            for i in reversed(middleware):
                run = i(run)

            async def wrapper(self, stream):
                stream.req_name = methodname
                return await run(stream)
            setattr(obj, methodname, wrapper)
            return wrapper
        return decorator




def patch_grpcname(protoname: str) -> tuple[str, str]:
    patch_grpcnameorigin = protoname
    if (patch_grpcnameorigin.endswith(".proto")):
        patch_grpcnameorigin = patch_grpcnameorigin[:-6]
    patch_grpcname = patch_grpcnameorigin + "_grpc.py"
    return patch_grpcname, patch_grpcnameorigin


def patch_pbname(protoname: str) -> tuple[str, str]:
    patch_pbnameorigin = protoname
    if (patch_pbnameorigin.endswith(".proto")):
        patch_pbnameorigin = patch_pbnameorigin[:-6]
    patch_pbname = patch_pbnameorigin + "_pb2.py"
    return patch_pbname, patch_pbnameorigin



from .proto import Proto
from .engine import GRPCLess
from .stream import Stream
from . import middleware
from .tools import main as cmd_tools

# from . import exceptions
try:
    from .import_fixer import fix
    fix()  # 修补模块导入
except ImportError:
    pass  # 处于生产模式，禁用导入修补


__all__ = [
    "Proto",
    "GRPCLess",
    "Stream",
    "middleware"
]
if __name__ == "__main__":
    cmd_tools.main()



# from .. import log
from . import compile
from . import import_pb
from . import utils
from typing import Any


try:
    from ..import_fixer import real_import
    DBG_DYNAMIC_IMPORT = True
except ImportError:
    DBG_DYNAMIC_IMPORT = False
    real_import = None


class Proto():
    def __init__(self,
                 *proto_files: str,
                 proto_path: str,
                 output_dir: str = 'pb',
                 output_type: list[str] = [
                     "python", "grpclib_python", "mypy"],
                 other_include: list[str] = [],
                 ):
        compile.build(
            *proto_files,
            proto_path=proto_path,
            output_dir=output_dir,
            output_type=output_type,
            other_include=other_include
        )
        self._output_dir = output_dir
        self._prod_dist = []
        import_pb.init_path(output_dir)
        import_protos: dict[str, tuple] = {}

        if (real_import is not None):
            real_import()

        for i in proto_files:
            import_protos[i] = import_pb.import_proto(output_dir, i)
            basename = utils.patch_pbname(i)[1]
            self._prod_dist.append((i, basename, import_protos[i][-1]))
            setattr(self, "pb_"+basename, import_protos[i][0])
            setattr(self, "grpc_"+basename, import_protos[i][1])
            setattr(self, "manifest_"+basename, import_protos[i][2])
        self.import_protos = import_protos

    def get_service(self, proto: str):
        protofile, service_name = proto.split(":")
        return getattr(self.import_protos[protofile][1], service_name+"Base")

    def __getattribute__(self, name: str) -> Any:
        return super().__getattribute__(name)



import os
import shutil
from .. import log


def copy_dist(*origin: str, dist: str = "dist"):
    for src in origin:
        if not os.path.exists(src):
            log.log_build(f"File not found: {src}")
            continue

        # 获取源文件/目录的名称
        src_name = os.path.basename(src)
        # 构建目标路径
        dst_path = os.path.join(dist, src_name)

        try:
            if os.path.isdir(src):
                # 如果是目录，使用shutil.copytree复制整个目录
                if os.path.exists(dst_path) and os.path.isdir(dst_path):
                    shutil.rmtree(dst_path)
                shutil.copytree(src, dst_path)
                log.log_build(f"Copy: {src} -> {dst_path}")
            else:
                # 如果是文件，使用shutil.copy2保留元数据
                shutil.copy2(src, dst_path)
                log.log_build(f"Copy: {src} -> {dst_path}")
        except Exception as e:
            log.log_build(f"Err: Copy: {src} -> {dst_path}")



from typing import Callable, Coroutine, Any
import time
import random
from .. import log


def LogMiddleware(func: Callable[[Any], Coroutine]) -> Callable:
    async def warpper(request) -> Coroutine:
        st = time.time()
        try:
            ret = await func(request)
        except Exception as e:
            ed = time.time()
            log.log_request(ed - st, request.__name__)
            raise e
        ed = time.time()
        log.log_request(ed - st, request.__name__)
        return ret
    return warpper


def LogStreamMiddleware(func: Callable[[Any], Coroutine]) -> Callable:
    async def warpper(request) -> Coroutine:
        st = time.time()
        stream_id = random.randint(1000, 9999)
        log.log_stream_start(request.req_name, stream_id)
        try:
            ret = await func(request)
        except Exception as e:
            ed = time.time()
            log.log_stream_stop(ed - st, request.req_name, stream_id)
            raise e
        ed = time.time()
        log.log_stream_stop(ed - st, request.req_name, stream_id)
        return ret
    return warpper



from datetime import datetime


def get_t() -> str:
    now = datetime.now()
    return now.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]


def format_time_duration(seconds: float) -> str:
    if seconds < 0:
        return f"-{format_time_duration(-seconds)}"

    if seconds >= 180:
        return f"{int(seconds)//60}m"
    if seconds >= 1:
        return f"{int(seconds)}s"
    milliseconds = seconds * 1000
    if milliseconds >= 1:
        return f"{int(milliseconds)}ms"
    microseconds = milliseconds * 1000
    if microseconds >= 1:
        return f"{int(microseconds)}μs"
    nanoseconds = microseconds * 1000
    return f"{int(nanoseconds)}ns"



import sys

from .color import *
from .time import get_t, format_time_duration
import time as sys_time


def print_logo() -> None:
    sys.stderr.write(FG_BLUE+" ==== grpcE Engine ==== "+RESET+"\n")


def log_compile(compile_file: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_MAGENTA}COMPILE{RESET}|{FG_GREY}   0ns{RESET}|Compile file "{compile_file}"\n')


def log_compile_global_fault(msg: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_MAGENTA}COMPILE{RESET}|{FG_GREY}   0ns{RESET}|{FG_RED}ERR{RESET}: {msg}\n')


def log_compile_file_fault(file: str, msg: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_MAGENTA}COMPILE{RESET}|{FG_GREY}   0ns{RESET}|{FG_RED}ERR{RESET}: {FG_GREY}{file}{RESET}|{msg}\n')


def log_import_access(path: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_MAGENTA}PROTO  {RESET}|{FG_GREY}   0ns{RESET}|Access output path: "{path}"\n')


def log_import_proto(file: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_MAGENTA}PROTO  {RESET}|{FG_GREY}   0ns{RESET}|Import file: "{file}"\n')


def log_start_server(host: str, port: int, loop: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|\n')
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|{FG_BLUE} ▄████ {RESET}| Python grpcLess Engine\n')
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|{FG_BLUE} █   ▄ {RESET}| {FG_GREY}Do less, Create more{RESET}\n')
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|{FG_BLUE} ▀████ {RESET}| ------------------\n')
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|{FG_BLUE} ▄   ▀ {RESET}| Loop Engine: {loop}\n')
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|{FG_BLUE} ▀████ {RESET}| Address: {FG_GREEN}{host}:{port}{RESET}\n')
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}SERVER {RESET}|{FG_GREY}   0ns{RESET}|\n')


def log_request(dur: float, name: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_GREEN}REQUEST{RESET}|{FG_GREY}{format_time_duration(dur):>6}{RESET}|Get "{name}"\n')


def log_stream_start(name: str, stream_id: int) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_GREEN}REQUEST{RESET}|{FG_GREY}   0ns{RESET}|Stream "{name}"{FG_GREY}[{stream_id}]{RESET} start\n')


def log_stream_stop(dur: float, name: str, stream_id: int) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_GREEN}REQUEST{RESET}|{FG_GREY}{format_time_duration(dur):>6}{RESET}|Stream "{name}"{FG_GREY}[{stream_id}]{RESET} stop\n')


def log_print(msg: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_GREY}DEBUG  {RESET}|{FG_GREY}   0ns{RESET}|{msg}\n')


def log_error(msg: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_RED}ERROR  {RESET}|{FG_GREY}   0ns{RESET}|{msg}\n')


def log_build_start() -> None:
    global st_time
    st_time = sys_time.time()
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}BUILD  {RESET}|{FG_GREY}   0ns{RESET}|Start build\n')


def log_build(msg: str) -> None:
    sys.stdout.write(
        f'{FG_GREY}{get_t()}{RESET}|{BG_BLUE}BUILD  {RESET}|{FG_GREY}{format_time_duration(sys_time.time()-st_time):>6}{RESET}|{msg}\n')



from grpclib.server import Stream as gStream
from typing import Any, Callable
from grpclib.exceptions import StreamTerminatedError


class Stream(gStream):
    def __init__(self, stream: gStream, fix_func: Callable = (lambda x: x), name: str = ""):
        self._fix_func = fix_func
        self._original_stream = stream
        self.req_name = name

    def __getattr__(self, name):
        return getattr(self._original_stream, name)

    def original_stream_mode(self):
        self.send_message = self.send_message_original

    async def send_message(self, message: Any) -> None:
        message = self._fix_func(message)
        return await super().send_message(message)

    async def send_message_original(self, message: Any) -> None:
        return await super().send_message(message)

    async def recv_message(self) -> Any | None:
        ret = await super().recv_message()
        if (ret is None):
            raise StreamTerminatedError()
        return ret

    def __aiter__(self):
        return self

    async def __anext__(self) -> Any:
        if (ret := await super().recv_message()) is None:
            raise StopAsyncIteration
        else:
            return ret

    async def cancel(self):
        return await super().cancel()

    async def close(self):
        return await super().__aexit__(None, None, None)


class ClientStream(Stream):
    async def send_message(self, message: Any) -> None:
        raise Exception("ClientStream can't send message")


class ServerStream(Stream):
    async def recv_message(self) -> Any | None:
        raise Exception("ServerStream can't recv message")

    async def __anext__(self) -> Any:
        raise Exception("ServerStream can't recv message")



import re
from typing import Any, Optional, Type


def extract_class_from_type_string(type_str: str, index: int = 0) -> list[str]:
    # 提取方括号中的内容
    match = re.search(r'\[(.*)\]', type_str)
    if not match:
        return []

    # 分割泛型参数
    generic_args = [str(arg.strip()) for arg in match.group(1).split(',')]

    if index >= len(generic_args):
        return []

    # 获取指定索引的参数
    class_path = generic_args[index]

    try:
        return class_path.rsplit('.', 1)
    except (ValueError, ImportError, AttributeError) as e:
        return []


def get_val_from_pb(pb: Any):
    return [tuple(i) for i in pb.__dict__["__init__"].__annotations__.items() if not i[0].startswith('__') and i[0] != "return"]
